import java.util.Arrays;

public class Search {

	/**
	 * Das Hauptprogram mit einigen Testfällen.
	 */
	public static void main(String[] args) {

		int data[];
		int value;
		SearchResult res;

		// ein paar Beispiele
		data = new int[] { 1, 2, 3, 4, 5, 6 };
		value = 4;
		conductTest(data, value);

		value = 0;
		conductTest(data, value);

		// Beispiel mit Zufallsdaten
		data = randomArray(20, false); // nicht sortiert
		value = data[2];
		conductTest(data, value);

		data = randomArray(20, true); // sortiert
		value = data[2];
		conductTest(data, value);
	}

	/**
	 * Eine Implementierung der linearen Suche als Refernzbeispiel.
	 * 
	 * @param data
	 *            die Daten innhalb welcher gesucht werden soll
	 * @param value
	 *            der Wert nach welchem gesucht werden soll
	 * @return der Index des Wertes innerhalb der gegebenen Daten oder -1 falls
	 *         nicht enthalten
	 */
	public static SearchResult linearSearch(int data[], int value) {
		SearchResult res = new SearchResult();

		// naive interative suche
		for (int i = 0; i < data.length; i++) {
			// überprüfen des aktuellen elements
			res.steps++;
			if (data[i] == value) {
				// index gefunden
				res.index = i;
				return res;
			}
		}
		// das gesuchte element ist nicht im array enthalten
		return res;
	}

	/**
	 * Eine Implementierung für eine Binäre Suche innerhalb sortierter Daten.
	 * 
	 * @param data
	 *            die sortierten Daten innhalb welcher gesucht werden soll.
	 * @param value
	 *            der Wert nach welchem gesucht werden soll
	 * @return der Index des Wertes innerhalb der gegebenen Daten oder -1 falls
	 *         nicht enthalten
	 */
	public static SearchResult binarySearch(int data[], int value) {
		/* TODO */
		return new SearchResult();
	}

	/**
	 * Eine Implementierung für eine Binäre Suche innerhalb sortierter Daten.
	 * 
	 * @param data
	 *            die sortierten Daten innhalb welcher gesucht werden soll.
	 * @param value
	 *            der Wert nach welchem gesucht werden soll
	 * @return der Index des Wertes innerhalb der gegebenen Daten oder -1 falls
	 *         nicht enthalten
	 */
	public static SearchResult interpolationSearch(int data[], int value) {
		/* TODO */
		return new SearchResult();
	}

	/**
	 * Vergleicht die drei Suchalgorithmen innerhalb dieses Beispiels beim
	 * suchen nach dem gegebenen Wert innerhalb der gegebenen Daten.
	 * 
	 * @param data
	 *            die Daten innhalb welcher gesucht werden soll
	 * @param value
	 *            der Wert nach welchem gesucht werden soll
	 */
	public static void conductTest(int data[], int value) {

		String names[] = { "linear", "binary", "interp" };
		SearchResult res[] = { linearSearch(data, value),
				binarySearch(data, value), interpolationSearch(data, value) };

		System.out.printf("Suche %d in %s ...\n", value, Arrays.toString(data));
		for (int i = 0; i < res.length; i++) {
			SearchResult cur = res[i];
			System.out.printf("  %s: %d \t Schritte: %d \t Korrekt: %b\n",
					names[i], cur.index, cur.steps, check(data, value, cur));
		}
		System.out.println();
	}

	/**
	 * Eine Hilfsmethode zum generieren von Arrays mit Zufallswerten im Bereich
	 * [0,100).
	 * 
	 * @param size
	 *            die Anzahl an Elementen im Resultierenden Array
	 * @param sorted
	 *            falls true, ist der resultierende Array sortiert, andernfalls
	 *            nicht
	 * @return der generierte Daten-Array
	 */
	public static int[] randomArray(int size, boolean sorted) {
		// anlegen der resultierenden Arrays
		int res[] = new int[size];

		// befüllen des Daten-Arrays mit Zufallsdaten
		for (int i = 0; i < size; i++) {
			res[i] = (int) (Math.random() * 100);
		}

		// sortieren - falls gewünscht
		if (sorted) {
			Arrays.sort(res);
		}
		return res;
	}

	/**
	 * Überprüft das Ergebnis einer Such-Operation.
	 * 
	 * @param data
	 *            die Daten innerhalb welche gesucht wurde
	 * @param value
	 *            der Wert nach welchem gesucht wurde
	 * @param res
	 *            das Such-Resultat welches überprüft werden soll
	 * @return true falls das Such-Resultat korrekt ist, false andernfalls
	 */
	public static boolean check(int data[], int value, SearchResult res) {
		// wurde der wert gefunden?
		if (res.index >= 0) {
			// überprüfe die gefunden position
			return data.length > res.index && data[res.index] == value;
		}

		// ist element tatsächlich nicht enthalten?
		for (int i = 0; i < data.length; i++) {
			if (data[i] == value) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Eine Hilfsklasse um Mehrere Daten aus einem Funktionsaufruf zu
	 * retunieren.
	 */
	static class SearchResult {
		public int index = -1;
		public int steps = 0;
	}

}
